/* Copyright (C) 2004-2016 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
 *
 * This file is part of datasid
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 */
#include <re2/re2.h>
#include <map>
#include <string>
#include "agent/linux/datasid-parse.h"

/* 
 * trim Function
 * Erases leading tabs, leading spaces; trailing tabs, trailing spaces. 
 */

void trim(std::string* str) {
    int i = 0;

    while (i < static_cast<int>(str->length()) && ((*str)[i] == ' '
        || (*str)[i] == '\t'))
        ++i;

    if (i > 0)
        str->erase(0, i);

    int j = i = str->length();

    while (i > 0 && ((*str)[i - 1] == ' ' || (*str)[i - 1] == '\t'))
        --i;

    if (i < j)
        str->erase(i, j);
}

/* 
 * normalize Function
 * Erases tabs and spaces between the variable's name and its value. 
 */

void normalize(std::string* str) {
    size_t i, j;
    i = j = 0;

    trim(str);

    while (i < str->length()) {
        if ((*str)[i] == ' ' || (*str)[i] == '\t') {
            j = i + 1;

            while (j < str->length() && ((*str)[j] == ' ' || (*str)[j] == '\t'))
                ++j;

            if (j == i)
                str->erase(i, 1);
            else
                str->erase(i, j - i);
        } else {
            ++i;
        }
    }
}

/* 
 * spaceOnly Functio
 * Check if a line consists only of spaces and tabs 
 */

bool spaceOnly(const std::string* line) {
    for (int i = 0, j = line->length(); i < j; ++i)
        if ((*line)[i] != ' ' && (*line)[i] != '\t')
            return false;
    return true;
}

/* 
 * isValid Function
 * Check if a line is valid
 * The line is valide when has "key":"value"; 
 */

bool isValid(std::string* line) {
    std::string s = *line;
    int i = 0;

    normalize(line);

    /* if the line is a comment */

    if (RE2::FullMatch(s, "((\\/\\/)|(\\/\\*)|(\\#))(.*)"))
        return false;

    /* if the line start with a different character than quote */

    if (!RE2::FullMatch(s,  "(\")(.*)"))
        return false;
    /* 
     * if the line start with quote, 
     * but is not a pattern : "key":"value" 
     */

    if ((*line)[i] = '"') {
        int n = std::count((*line).begin(), (*line).end(), '"');

        /* if the line doesn't has four quotes */
        if (n < 4)
            return false;

        /* if the second character is quote. */

        if ((*line)[i+1] == '"')
            return false;

        n = line->find_last_of('"');

        /* if the second to last character is quote. */

        if ((*line)[n-1] == '"')
            return false;

        n = line->find_last_of(':');

        /* if the line doesn't has the colon character. */

        if (n == -1)
            return false;

        /* if the position of the colon character is string ends. */

        if (n == (line->length())-1)
            return false;
    }

    return true;
}

/*
 * parse Function
 * After check if the line is valid,
 * than parse the "key":"value" to a map.
 */

void parse(std::string* line, std::map<std::string, std::string>* content) {
    std::locale loc;

    /* if the line is a "key":"value" */

    if ((*line)[0] == '"') {
        std::string name = line->substr(1, line->find(':')-2);

        for (int i = 0; i < name.length(); i++)
            name[i] = toupper(name[i], loc);

        std::string value = line->substr(line->find(':')+2);
        value = value.substr(0, value.find('"'));

        (*content)[name] = value;
    }
}

/*
 * readFile Function
 * read a file (config or proxy file)
 * Than check if the line is valid.
 * Case is valid, parse the "key":"value" to a map.
 * After all, get de value by key and "initialize"
 * a conf or proxy object.
 */

bool readFile(const std::string file, std::map<std::string,
    std::string>* content) {

    content->clear();

    std::ifstream config(file.c_str(), std::ifstream::in);

    if (!config.is_open())
        return false;

    std::string buffer;

    while (getline(config, buffer, '\n')) {
        if (!spaceOnly(&buffer))
            if (isValid(&buffer))
                parse(&buffer, content);
    }

    return true;
}
