#!/bin/bash

# allow for improved pattern recognition
shopt -s extglob

# is docker installed?
command -v docker >/dev/null 2>&1 || { echo >&2 "It seems docker is not installed. Aborting."; exit 1; }


# Check options
# -c - rsync agent directory into image directory
# -b - build only specified image / all images if no argument is provided
# -r - run only specified image / all images if no argument is provided
# -q - redirect stdout of rsync and docker build to specified file / log.txt if
#      no argument is provided

LINT=true
EXECUTE_ALL=true
RSYNC=false
BUILD_DOCKER_IMAGES=false
RUN=false
QUIET_MODE=false

# current tested images
declare -A IMAGES_LIST=([ubuntu]=1 [debian]=1 [opensuse]=1)

while (($#))
do
    case "$1" in
        -c)
            RSYNC=true
            EXECUTE_ALL=false
            shift 1
            ;;

        -b)
            BUILD_DOCKER_IMAGES=true
            EXECUTE_ALL=false
            if [[ "$2" = -* ]] || [[ "$2" = "" ]]
            then
                # no arguments
                shift 1
            else
                if [[ -n "${IMAGES_LIST[$2]}" ]]
                then
                    B_IMAGE_NAME=$2
                else
                    echo "Invalid image name: $2" >&2
                fi
                shift 2
            fi
            ;;

        -r)
            RUN=true
            EXECUTE_ALL=false
            if [[ "$2" = -* ]] || [[ "$2" = "" ]]
            then
                # no arguments
                shift 1
            else
                if [[ -n "${IMAGES_LIST[$2]}" ]]
                then
                    R_IMAGE_NAME=$2
                else
                    echo "Invalid image name: $2" >&2
                fi
                shift 2
            fi
            ;;

        -q)
            QUIET_MODE=true
            if [[ "$2" = -* ]] || [[ "$2" = "" ]]
            then
                # no arguments
                shift 1
            else
                LOG_FILE_NAME=$2
                shift 2
            fi
            ;;

        \?)
            echo "Invalid option: $2" >&2
            shift
            ;;
    esac
done


# If no options were used, run everything
if ($EXECUTE_ALL)
then

    RSYNC=true
    BUILD_DOCKER_IMAGES=true
    RUN=true

fi

# If quiet mode was selected, redirect stdout to log.txt file
if ($QUIET_MODE)
then
    # save stdout
    exec 5<&1
    # redirect
    if [ -z "${LOG_FILE_NAME}" ]
    then
        exec 1> log.txt
    else
        exec 1> "${LOG_FILE_NAME}"
    fi
fi

# rsync
if ($RSYNC)
then
    printf "******************\n*  LINT and COPY *\n******************\n\n"

    printf "Applying linter to /agent/src/*.cpp\n"
    cpplint.py --filter=-whitespace/blank_line,-build/include_what_you_use ../src/!(jsoncpp).cpp ../src/linux/*.cpp ../src/windows/*.cpp
    shopt -u extglob

    for i in "${!IMAGES_LIST[@]}"
    do
        printf "Copying to docker/%s/agent\n" "$i"
        rsync -au ../* docker/"$i"/agent --delete --exclude test --exclude build/*
        printf "Copying to docker/%s/create_users.sh\n\n" "$i"
        rsync -u create_users.sh docker/"$i"/create_users.sh

    done

    printf "\n"

fi


# build images
if ($BUILD_DOCKER_IMAGES)
then
    printf "***********\n*  BUILD  *\n***********\n\n"

    if [ -z "${B_IMAGE_NAME}" ]
    then
        for i in "${!IMAGES_LIST[@]}"
        do
            printf "Building %s:agent\n\n" "$i"
            docker build -t "$i":agent docker/"$i"
            printf "\n"
        done
    else
        printf "Building %s:agent\n\n" "${B_IMAGE_NAME}"
        docker build -t "${B_IMAGE_NAME}":agent docker/"${B_IMAGE_NAME}"
        printf "\n"
    fi

    printf "\n"

fi

# If quiet mode was selected, restore stdout before running
if ($QUIET_MODE)
then
    exec 1<&5
fi

# run agent into images
if ($RUN)
then
    printf "*********\n*  RUN  *\n*********\n"

    if [ -z "${R_IMAGE_NAME}" ]
    then
        for i in "${!IMAGES_LIST[@]}"
        do
            printf "\nRunning %s:agent\n" "$i"
            printf "******************************************************\n"
            docker run -it "$i":agent ./home/agent/bin/agent-v0.0
            printf "******************************************************\n\n"
        done
    else
        printf "\nRunning %s:agent\n\n" "${R_IMAGE_NAME}"
        printf "******************************************************\n"
        docker run -it "${R_IMAGE_NAME}":agent ./home/agent/bin/agent-v0.0
        printf "******************************************************\n"
    fi

fi
